--------------------------------------------------
---  Copyright 2008-2019, HDRsoft - www.hdrsoft.com
--------------------------------------------------

local LrApplication = import 'LrApplication'
local LrDialogs = import 'LrDialogs'
local LrErrors = import 'LrErrors'
local LrDate = import 'LrDate'
local LrFileUtils = import 'LrFileUtils'
local LrPathUtils = import 'LrPathUtils'
local LrExportSession = import 'LrExportSession'
local LrShell = import 'LrShell'
local LrTasks = import 'LrTasks'
local LrLogger = import 'LrLogger'
local prefs = import 'LrPrefs'.prefsForPlugin()


BatchPhotomatixCom = {}


local appPath

function BatchPhotomatixCom.initAppPath()

	local pathExist=false

  if WIN_ENV then
  	if prefs.isDebug then
        appPath="H:\\Projects\\Photomatix\\Photomatix\\bin\\x64\\Debug\\PhotomatixClient.exe"
  	else
  		appPath = LrPathUtils.child(_PLUGIN.path,"PhotomatixClient.exe")
  	end
  else --MAC_ENV
    local baseAppPath = "/Applications"
    local pmpAppPath = "Photomatix Pro 6.app"
    local contAppPath = "Contents"
    local resAppPath = "Resources"
    local clientName = "PhotomatixClient"
    appPath=LrPathUtils.child(baseAppPath, LrPathUtils.child(pmpAppPath, LrPathUtils.child(contAppPath, LrPathUtils.child(resAppPath, clientName))))
    --appPath=LrPathUtils.child(_PLUGIN.path,LrPathUtils.child(pmpAppPath,LrPathUtils.child(contAppPath,LrPathUtils.child(resAppPath,clientName))))

  end
	require 'PhotomatixApp'

	if appPath then
		pathExist=LrFileUtils.exists(appPath)
		if pathExist then
			pathExist=PhotomatixApp.clientExists(appPath, true)
		else
			BatchUploadTask.outputToLog(string.format('LrFileUtils.exists returned false for: %s',appPath))
		end
	end

	if not pathExist then
		LrErrors.throwUserError(LOC "$$$/Photomatix/Batch/Message/NotFoundPMP=Could not find the Photomatix application that lets you choose a Preset with preview. Please try reinstalling the Plugin.")
			return
	end
end

local currentPresetToPreview
local currentDoCreatePreset
local currentXmpOutDir
local currentPropertiesToUse

function BatchPhotomatixCom.callPreviewForPreset(nPhotos, doCreatePreset, presetToPreview, props)
	currentXmpOutDir = nil

	BatchUploadTask.outputToLog("Entering callPreviewForPreset", true)

	local previewIsForStack = false
	if nPhotos <=0 then
		previewIsForStack = true
	end

	local tempPath="D:\\Images\\Samples\\Tmp"

	local activeCatalog = LrApplication.activeCatalog()


	-- local nPhotos = exportSession:countRenditions()

	-- local progressScope = exportContext:configureProgress{
						-- title = nPhotos > 1
									-- and LOC( "$$$/Photomatix/Upload/Progress=Exporting ^1 photos to Photomatix Pro", nPhotos )
									-- or LOC "$$$/Photomatix/Upload/Progress/One=Exporting one photo to Photomatix Pro",
					-- }

	local files = {}
	local nFiles=0
	local inputFilesArgs=""
	local photos2Export = {}

	for iPhoto, photo in ipairs(activeCatalog.targetPhotos) do
		local skipPhoto = previewIsForStack and not photo:getRawMetadata("isInStackInFolder")
		if not skipPhoto then
			if not previewIsForStack then
				table.insert(photos2Export, photo)
				nFiles = nFiles + 1

				if nFiles == nPhotos then
					break
				end
			else
				local stackedPhotos=photo:getRawMetadata("stackInFolderMembers")
				for dummy,sPhoto in pairs(stackedPhotos) do
					table.insert(photos2Export, sPhoto)
				end
				nPhotos = #stackedPhotos
				nFiles = nPhotos --nPhotos and nFiles not used below but setting them to correct value, for stack case, just in case we want to use them later on.
				break
			end
		end
	end

				local newExportSession = LrExportSession{
					exportSettings = {
						LR_exportServiceProvider = 'com.hdrsoft.photomatix.batch.lightroom',
						LR_exportServiceProviderTitle = "Photomatix",
						LR_format = "JPEG",
						LR_tiff_compressionMethod = "compressionMethod_None",
						LR_export_bitDepth = 8,
						LR_export_colorSpace = "AdobeRGB",
						LR_minimizeEmbeddedMetadata = false,
						LR_size_resolution = 300,
						LR_size_doConstrain = true,
						LR_size_doNotEnlarge = true,
						LR_size_resizeType = "megapixels",
						LR_size_megapixels = 2.7,

						LR_metadata_keywordOptions = "lightroomHierarchical",
						-- LR_minimizeEmbeddedMetadata = false,
						--- LR_removeLocationMetadata=false,
					},
					photosToExport = photos2Export
				}
				currentPresetToPreview=presetToPreview
				currentDoCreatePreset=doCreatePreset
				currentPropertiesToUse = props
				prefs.isPreviewPreset=true
				newExportSession:doExportOnCurrentTask()
			-- end )

	return currentXmpOutDir

end

function BatchPhotomatixCom.ExportImagesForPreview(exportContext)
	BatchUploadTask.outputToLog("Entering ExportImagesForPreview", true)
	local inputFilesArgs=""
	local tempPath=BatchUploadTask.getTempPath()
	local srcPreviewPhotos = {}
	for i, rendition in exportContext:renditions() do
		local success, pathOrMessage = rendition:waitForRender()

		if success then

			local copyPath = LrPathUtils.child(tempPath, LrPathUtils.leafName(pathOrMessage))
			LrFileUtils.copy(pathOrMessage, copyPath)
			table.insert(srcPreviewPhotos, copyPath)
			inputFilesArgs=string.format('%s \"%s\"',inputFilesArgs,copyPath)
		-- end
		else
			LrErrors.throwUserError(LOC "$$$/Photomatix/Batch/Message/ErrorBuildingPreview=The preview could not be displayed.")
			---	error(pathOrMessage)
		end
	end
	local presetToPreview=currentPresetToPreview
	local doCreatePreset=currentDoCreatePreset
	local nFiles=#srcPreviewPhotos
	local props=currentPropertiesToUse
	BatchUploadTask.outputToLog(string.format("nFiles to preview=%d",nFiles))

	BatchPhotomatixCom.initAppPath()

	if nFiles > 0 then
		local doesExist=LrFileUtils.exists(appPath)
		BatchUploadTask.outputToLog(string.format("Exists=%s appPath=%s",doesExist,appPath))
		if doesExist then

			local PhotomatixClientCommand=string.format('\"%s\"', appPath)

            if WIN_ENV then
			   PhotomatixClientCommand=string.format('%s -pi LrBatchAutomation -h -p \"%s\"', PhotomatixClientCommand, presetToPreview)
            end
            if MAC_ENV then
				PhotomatixClientCommand=string.format('%s -p \"%s\"',PhotomatixClientCommand, presetToPreview)
			end

			-- local ldrBits=8
			-- if prefs.outputFormat=='tif16' then
				-- ldrBits=16
			-- end
			-- PhotomatixClientCommand=string.format('%s -bi %d', PhotomatixClientCommand, ldrBits)
			if nFiles>1 then
				if props.chkAlign then
					local alignType = 2
					if props.presetAlign == "1" then
						alignType = 1
					end
					if props.chkCrop then
						PhotomatixClientCommand=string.format('%s -a%dc', PhotomatixClientCommand, alignType)
					else
						PhotomatixClientCommand=string.format('%s -a%dn', PhotomatixClientCommand, alignType)
					end
					if props.presetAlign == "0" or props.presetAlign== "2" then
						PhotomatixClientCommand=string.format('%sp', PhotomatixClientCommand)
						if props.presetAlign == "0" then --set maxShift to 20
							PhotomatixClientCommand=string.format('%s20', PhotomatixClientCommand)
						end
					elseif props.presetAlign == "3" then --correction for rotation only
						PhotomatixClientCommand=string.format('%sn', PhotomatixClientCommand)
					end

					-- PhotomatixClientCommand=string.format('%s%s', PhotomatixClientCommand, prefs.maxShift)
				end

				if props.chkGhosting then
					PhotomatixClientCommand=string.format('%s -g2n', PhotomatixClientCommand)
				end
			end

			local xmpOutDir=BatchUploadTask.getTempPath()

			if doCreatePreset then


				PhotomatixClientCommand=string.format('%s -w', PhotomatixClientCommand)
				PhotomatixClientCommand=string.format('%s -o \"%s\"', PhotomatixClientCommand,xmpOutDir)
			end

			if MAC_ENV then	-- we add the path of the folder containing the exported files to the arguments passed to PhotomatixClient
				PhotomatixClientCommand=string.format('%s -fo \"%s\"',PhotomatixClientCommand, tempPath)
			end
			local outputTempFile=string.format('%sCmdOutput', tempPath)
			if (WIN_ENV) then
				PhotomatixClientCommand=string.format('%s %s %s > \"%s\"', PhotomatixClientCommand, "ToneMap", inputFilesArgs, outputTempFile)
				PhotomatixClientCommand=string.format('\"%s\"', PhotomatixClientCommand)
			else
				PhotomatixClientCommand=string.format('%s %s %s', PhotomatixClientCommand, "SetPresetForBatch", inputFilesArgs)
			end

			BatchUploadTask.outputToLog(string.format('PhotomatixClientCommand: %s', PhotomatixClientCommand))

			if (WIN_ENV) then
				LrFileUtils.delete(outputTempFile) -- in case it is there from before
			end

			local retour=LrTasks.execute(PhotomatixClientCommand)

			if pmdebug then
			BatchUploadTask.outputToLog(string.format('retour PhotomatixClient: %d', retour))
			end
			if doCreatePreset then
				-- return xmpOutFile
				currentXmpOutDir = xmpOutDir
			end

			if (WIN_ENV) then
				LrFileUtils.delete(outputTempFile)
			end
		end
	end
end


