-------------------------------------------
---  Copyright HDRsoft - www.hdrsoft.com
-------------------------------------------

local LrApplication = import 'LrApplication'
local LrDialogs = import 'LrDialogs'
local LrFunctionContext = import 'LrFunctionContext'
local LrErrors = import 'LrErrors'
local LrDate = import 'LrDate'
local LrFileUtils = import 'LrFileUtils'
local LrPathUtils = import 'LrPathUtils'
local LrExportSession = import 'LrExportSession'
local LrProgressScope = import 'LrProgressScope'
local LrShell = import 'LrShell'
local LrTasks = import 'LrTasks'
local LrLogger = import 'LrLogger'
local prefs = import 'LrPrefs'.prefsForPlugin()

local myLogger = nil

BatchUploadTask = {}

local machID

function BatchUploadTask.outputToLog(message, skipInProd)
	if myLogger == nil then
		myLogger = LrLogger('PhotomatixBatchLogger')
		myLogger:enable("print")
		if prefs.isDebug and not MAC_ENV then
			myLogger:enable("logfile")
		end
	end
	if skipInProd and not prefs.isDebug then
		return
	end
	myLogger:trace(message)
end

--[[.#luaCloseComment#
prefs.isDebug = false
--]]
--.#luaOpenBrackets#
prefs.isDebug = true
--]]

BatchUploadTask.outputToLog(string.format("BatchUploadTask started, isDebug=%s", prefs.isDebug))

function BatchUploadTask.getBatchPath(exeName)
  local appPath
  if WIN_ENV then
  	if prefs.isDebug then
  		appPath = "D:\\Prog\\Dev\\PMP6\\Photomatix\\bin\\x86\\Debug\\" .. exeName .. ".exe"
  	else
  		appPath = LrPathUtils.child(_PLUGIN.path,exeName .. ".exe")
  	end
  else -- MAC_ENV
    appPath = LrPathUtils.child(LrPathUtils.child(_PLUGIN.path,exeName),exeName)
--gj200913 - if PhotomatixCL not in plugin's folder, look for it in the PMP app package installed in the Applications folder
    if not LrFileUtils.exists(appPath) then
        local clPath=LrPathUtils.child("/Applications", LrPathUtils.child("Photomatix Pro 6.app", LrPathUtils.child("Contents", LrPathUtils.child("Resources", exeName))))
        appPath=LrPathUtils.child(clPath, exeName)
    end
-- gj200913 - end
  end
  if not LrFileUtils.exists(appPath) then
    appPath = false
  end
  return appPath
end

function BatchUploadTask.getBatchPathCL()
  return BatchUploadTask.getBatchPath("PhotomatixCL")
end

function BatchUploadTask.getBatchPathUtil()
  if WIN_ENV then
    return BatchUploadTask.getBatchPath("Merge32Utils")
  else
    return BatchUploadTask.getBatchPathCL()
  end
end

local appPath = BatchUploadTask.getBatchPathCL()
--if WIN_ENV then
--	if prefs.isDebug then
--		appPath = "D:\\Prog\\Dev\\PMP6\\Photomatix\\bin\\x86\\Debug\\PhotomatixCL.exe"
--	else
--		appPath = LrPathUtils.child(_PLUGIN.path,"PhotomatixCL.exe")
--	end
--else
--  --appPath="/Applications/PhotomatixCL/PhotomatixCL"
--  appPath = LrPathUtils.child(LrPathUtils.child(_PLUGIN.path,"PhotomatixCL"),"PhotomatixCL")
--end
--local appPath="D:\\Prog\\Dev\\PMP6\\Photomatix\\bin\\x86\\Debug\\PhotomatixCL.exe"
--local appPath="/home/francois/dev/Mac/PhotomatixCL/PhotomatixCL_1.0Beta1/PhotomatixCL"
--local appPath="/Applications/PhotomatixCL/PhotomatixCL"
local tempPath = nil
local outputTempFile

-- local iBidon=0
-- function BatchUploadTask.shouldRenderPhoto( exportSettings, photo)
	-- LrDialogs.message("In shouldRenderPhoto! :)")
	-- iBidon=iBidon+1
	-- return iBidon%2==0
-- end

function BatchUploadTask.getTempPath()
	local sysTempPath = LrPathUtils.getStandardFilePath('temp')
	-- if prefs.isDebug then
		-- if WIN_ENV then -- local dev folders on FJ machine (comment or adapt when running in development)
			-- sysTempPath="D:\\Images\\Samples\\Tmp"
		-- else
			-- sysTempPath="/tmp/tmpLR"
		-- end
	-- end
	local tempPath -- Local so that it does not overlap class variable with same name
	repeat
		local now = LrDate.currentTime()
		tempPath = LrPathUtils.child(sysTempPath, LrDate.timeToUserFormat(now, 'temp_lrexport2photomatix_%m%d%H%M%S'))
		if LrFileUtils.exists(tempPath) then
			tempPath = nil
		else
			LrFileUtils.createAllDirectories(tempPath)
		end
	until tempPath
	BatchUploadTask.outputToLog(string.format('tempPath: %s',tempPath), true)
	return tempPath
end

function BatchUploadTask.processRenderedPhotos( functionContext, exportContext )

	BatchUploadTask.outputToLog('Start BatchUploadTask.processRenderedPhotos', true)

	local exportSession = exportContext.exportSession
	local activeCatalog = LrApplication.activeCatalog()

	local lrVersion=LrApplication.versionTable
	if lrVersion then
		prefs.LrVersion=lrVersion().major
	else
		prefs.LrVersion=1
	end

    local txtMessageTitle=""
    local txtMessageDescr=""

-- gj200810 - Added code below to return an error when just one photo selected, but now commenting it out for the time being because:
-- The code doesn't differentiate when one stack is selected
-- The check is for a scenario that is unlikely to happen as it's only with the 'Unstacked with <NumberOfPhotos>' option, which is the non-default option and doesn't accept 1 as number of photos
--[[
    local cntPhotos=0
    activeCatalog:withReadAccessDo( function()
        local selectedPhotos=activeCatalog.targetPhotos
        cntPhotos=#selectedPhotos
	end )
    if cntPhotos==1 then
        txtMessageTitle = LOC "$$$/Photomatix/Batch/Message/InputNoBrackets/Title=No images to merge"
        txtMessageDescr = LOC "$$$/Photomatix/Batch/Message/InputNoBrackets/Description=The HDR batch merges bracketed exposures but there is only one photo selected.\n\nPlease select stacks or sets of bracketed photos and relaunch the plugin."
        LrDialogs.message(txtMessageTitle, txtMessageDescr)
        return
    end
--]]

	if WIN_ENV then
		-- Get machine ID that we need for generating key to pass to PhotomatixCL later.  It's fetched here for obfuscation purposes.
		local appPath=BatchUploadTask.getBatchPathUtil()
		if (appPath) then
			local CommandLine=string.format('\"%s\" -kh ', appPath)
			CommandLine=string.format('\"%s\"', CommandLine)
			BatchUploadTask.outputToLog(string.format('command is  %s', CommandLine))
			machID=LrTasks.execute(CommandLine)
			BatchUploadTask.outputToLog(string.format('After LrTasks.execute - retour is: %s', machID))
		end
	end

    BatchUploadTask.outputToLog(string.format("LrVersion=%s.%s.%s.%s.%s prefs=%s",lrVersion().major,lrVersion().minor,lrVersion().revision,lrVersion().build,lrVersion().publicBeta,prefs.LrVersion))


	require 'BatchInputSettings'
	require 'BatchRegistration'
	if true then
		-- BatchUploadTask.outputToLog(string.format("AppData: %s",LrPathUtils.getStandardFilePath("appData") ))
		-- BatchUploadTask.outputToLog(string.format("Prefs: %s",LrPathUtils.getStandardFilePath("appPrefs") ))
		-- LrDialogs.message(string.format("number of renditions: %d",exportSession:countRenditions()))
		-- return
	end

	local isPreview=false
	if prefs.isPreviewPreset then
		prefs.isPreviewPreset=false
		isPreview=true
	end

	if isPreview then
		-- LrDialogs.message(string.format("Would export there, but exiting for now: %d", exportSession:countRenditions()))
		BatchPhotomatixCom.ExportImagesForPreview(exportContext)
		return
	end

	local sessionPass = 1
	if (prefs.sessionPass == 2) then
		prefs.sessionPass = 1
		sessionPass = 2
	end


	BatchUploadTask.outputToLog(string.format("Dumping properties, for pass %s:",sessionPass))
	for ip, param in exportContext.propertyTable:pairs() do
		BatchUploadTask.outputToLog(string.format("Param[%s]=%s", ip, param ))
	end

	prefs.hasStacks = false
	prefs.hasUnstacked = false
	prefs.firstIsStack = false
	prefs.firstStack = nil
	for iph,photo in pairs(activeCatalog.targetPhotos) do
		if photo:getRawMetadata("isInStackInFolder") then
			prefs.hasStacks = true
			if prefs.firstStack == nil then
				prefs.firstStack = photo:getRawMetadata("stackInFolderMembers")
				if not prefs.hasUnstacked then
					prefs.firstIsStack = true
				end
			end
		else
			prefs.hasUnstacked = true
		end
	end
	-- LrDialogs.message(string.format("HasStacks=%s",prefs.hasStacks))

	-- if not prefs.reprocessInStacks then

	prefs.batchStacks = prefs.batchStacks and not prefs.doBatchStacks

	if sessionPass == 1 then -- Showing plugin dialog and getting the options selected
		local doProcess=BatchInputSettings.getOptions()

        -- checking consistency between Input selection and presence of Stacks
        if prefs.doBatchStacks and not prefs.hasStacks then
            txtMessageTitle = LOC "$$$/Photomatix/Batch/Message/InputNoStacks/Title=No Stacks selected"
            txtMessageDescr = LOC "$$$/Photomatix/Batch/Message/InputNoStacks/Description=The batch couldn't proceed as none of the photos you selected are grouped into Stacks.\n\nPlease either group your bracketed exposures into Stacks before invoking the plugin or choose the option to batch unstacked bracketed photos (2nd radio button in the Input section)."
            LrDialogs.message(txtMessageTitle, txtMessageDescr)
            return
        end

		if doProcess and not BatchRegistration.CheckRegistration() then
			return
		end

		if not doProcess then
			-- LrDialogs.message(string.format("Exiting doProcess=%s",doProcess))
			return
		end

		exportContext.propertyTable.LR_export_colorSpace=prefs.colorProfile
		LrTasks.startAsyncTask( function()


			local isFusionWithStacksAndSingle = prefs.doBatchStacks and prefs.hasStacks and prefs.hasUnstacked and BatchInputSettings.isPresetForFusion(prefs.presetToUse)

			local processStacks = prefs.doBatchStacks and prefs.hasStacks
			local photos2Export={}
			local treatedTopPhotoIds={}
			for photo in exportSession:photosToExport() do
				local isInStack = photo:getRawMetadata("isInStackInFolder")
				if processStacks and isInStack then

					local topPhoto = photo:getRawMetadata("topOfStackInFolderContainingPhoto")
					local topPhotoId = topPhoto.localIdentifier
					if not treatedTopPhotoIds[topPhotoId] then
						treatedTopPhotoIds[topPhotoId] = true
						local stackedPhotos=photo:getRawMetadata("stackInFolderMembers")
						for k,sPhoto in pairs(stackedPhotos) do
							table.insert(photos2Export, sPhoto)
						end
					end
				-- elseif not isFusionWithStacksAndSingle then
					-- table.insert(photos2Export, photo)
				else--if (not prefs.doBatchStacks) and (not isInStack) then
					table.insert(photos2Export, photo) --single photo processing not happening anymore so photo will in fact be skipped in sessionPass 2
				end
			end
			BatchUploadTask.outputToLog("Dumping properties, of incoming session", true)
			local exportParams={}
			for ip, param in exportContext.propertyTable:pairs() do
				BatchUploadTask.outputToLog(string.format("Param[%s]=%s", ip, param ), true)
				exportParams[ip]=param
			end
			exportParams["LR_exportServiceProvider"] = "com.hdrsoft.photomatix.batch.lightroom" --hard-coding here because this is not set when plugin is invoked from the  contextual menu, for some reason.
			local newExportSession = LrExportSession {
				photosToExport = photos2Export,
				exportSettings = exportParams -- exportContext.propertyTable,
			}
			prefs.sessionPass = 2
			newExportSession:doExportOnCurrentTask()
		end )
		return --we exit this sessions which will be "restarted" (with sessionPass == 2) by the above

	end

	-- At this point we're in the case sessionPass == 2 which means the final session where the batch is actually running


	local doesExist=LrFileUtils.exists(appPath)
	BatchUploadTask.outputToLog(string.format("AppExists=%s path=%s",doesExist,appPath))

	if not doesExist then
		LrErrors.throwUserError(LOC "$$$/Photomatix/Batch/Message/NotFoundPhotomatixCL=Could not find the Photomatix HDR Batch utility. Please try reinstalling the Plugin.")
		return --needed?
	end


	local txtProgress=LOC "$$$/Photomatix/Batch/Progress/ExportPhotos=Initializing export..."
	-- local nFiles=0
	local retour=-1
	-- local tempPath = nil
	local photoToStackWith=nil
	local pathPhotoToStackWith=nil
	local photosProcessed=false

	-- BatchUploadTask.outputToLog(string.format("\n\nSetting pmpclArgs to nil, it currently is: %s\n\nn",pmpclArgs), true)
	pmpclArgs=nil -- So that batch doesn't reuse the same arguments to PhotomatixCL as before when it is run again (LRPIBATCH-15)

	LrFunctionContext.callWithContext('ExecuteBatchProcess', function(context)

		-- local outerProgressScope = LrDialogs.showModalProgressDialog({
					-- title = LOC "$$$/Photomatix/Batch/Progress/Batching=Batching...",
					-- caption = txtProgress,
					-- cannotCancel = true,
					-- functionContext = context,
				-- })
		local outerProgressScope = exportContext:configureProgress({
					title = LOC "$$$/Photomatix/Batch/Progress/Batching=Batching...",
					-- renderPortion = 0.
					caption = txtProgress,
					cannotCancel = false,
					-- functionContext = context,
				})

		local nbToTreat=0
		-- local targetPhotosId={}

		-- for iPhoto,photo in ipairs(activeCatalog.targetPhotos) do
			-- if prefs.batchStacks then
				-- stackedPhotos=photo:getRawMetadata("stackInFolderMembers")
				-- for k,sPhoto in pairs(stackedPhotos) do
					-- table.insert(targetPhotosId, sPhoto.localIdentifier)
				-- end
			-- else
				-- table.insert(targetPhotosId, photo.localIdentifier)
			-- end
		-- end

		-- for i, rendition in exportSession:renditions() do
			-- -- if iBidon%2==0 then
				-- -- rendition:skipRender()
			-- -- end
			-- -- iBidon=iBidon+1
			-- local shouldSkipPhoto=true
			-- -- if prefs.batchStacks then

			-- -- end
			-- -- else
				-- -- for iPhoto,photoId in ipairs(activeCatalog.targetPhotos) do
				-- for iPhoto,photoId in ipairs(targetPhotosId) do
					-- if photoId==rendition.photo.localIdentifier then
						-- -- rendition:skipRender()
						-- shouldSkipPhoto=false
						-- nbToTreat=nbToTreat+1
						-- break
					-- end
				-- end
			-- -- end
			-- if shouldSkipPhoto then
				-- rendition:skipRender()
			-- end
		-- end

		nbToTreat = exportSession:countRenditions()
		-- local presetPath = "D:\\Prog\\Dev\\PMP6\\PhotomatixSetup\\BuiltinPresets\\CompressorDeep.xmp"
		-- local presetPath = LrPathUtils.child(_PLUGIN.path,LrPathUtils.child("presets",LrPathUtils.addExtension(prefs.builtinPreset, "xmp")))
--		local presetPath = prefs.builtinPreset
		local presetPath = prefs.presetToUse


		-- If the user is viewing a (non-smart) collection, keep track of what it is, so we can
		-- add the output photo to the collection too.
		local sourceCollection = nil

		if prefs.LrVersion > 2 then  -- gj150417: added conditioning to avoid error when using the Plugin in LR 2
			for i, v in ipairs( activeCatalog:getActiveSources() ) do
				if v ~= nil and v.type ~= nil then
					if v:type() == "LrCollection" then
						if not v:isSmartCollection() then
							sourceCollection = v
						end
					end
				end
			end
		end


		local errorMessages = {}
		local skipProcessing = false
		local nbSkipped = 0
		local nbTreated = 0
		local nbSuccess = 0
		local nbBracketSuccess = 0
		local nbSingleSuccess = 0
		local nbErrors = 0
		local nbBracketErrors = 0
		local nbSingleErrors = 0
		local files = {}
		local nFiles = 0
		local kMax=prefs.kMax
		-- if kMax>nbToTreat then
			-- kMax=nbToTreat
		-- end
		local srcImagesFolder = nil
		-- if prefs.batchStacks then
			-- kMax=0
		-- end
		if prefs.doBatchStacks then
			kMax=1
		end
		local topStackPhoto=nil
		BatchUploadTask.outputToLog(string.format("About to enter rendition loop with kMax=%d, nbToTreat=%d",kMax,nbToTreat))
		for i, rendition in exportContext:renditions{ stopIfCanceled = true } do
			local isInStack = rendition.photo:getRawMetadata("isInStackInFolder")
			if not rendition.wasSkipped then
				outerProgressScope:setCaption(string.format("Exporting =%s",LrPathUtils.leafName(rendition.destinationPath)))
				-- if prefs.batchStacks and kMax<1 then
					-- kMax=rendition.photo:getRawMetadata("countStackInFolderMembers")
				-- end
				if not srcImagesFolder then
					srcImagesFolder = LrPathUtils.parent(rendition.photo.path)
				end

				-- local isInStack = rendition.photo:getRawMetadata("isInStackInFolder")
				if prefs.doBatchStacks and isInStack then
					if kMax == 1 then
						kMax=rendition.photo:getRawMetadata("countStackInFolderMembers")
						topStackPhoto=rendition.photo:getRawMetadata("topOfStackInFolderContainingPhoto")
						if topStackPhoto==nil then
							BatchInputSettings.outputToLog(string.format("TopPhoto returned null: %s",rendition.destinationPath))
						end
					end
					if not (topStackPhoto.localIdentifier==rendition.photo:getRawMetadata("topOfStackInFolderContainingPhoto").localIdentifier) then
						BatchInputSettings.outputToLog(string.format("Warning got photo from another stack curPhoto=%s topPhoto=%s",rendition.destinationPath,topStackPhoto:getRawMetadata("path")))
					end
					if prefs.chkStack and photoToStackWith == nil then
						photoToStackWith = rendition.photo
					end
				end
			end

			local success, pathOrMessage = rendition:waitForRender()

			if outerProgressScope:isCanceled() then break end

			if success then
				-- outerProgressScope:setCaption(string.format("Importing %d skipped=%s named: %s",i,rendition.wasSkipped, LrPathUtils.leafName(pathOrMessage)) )
				-- if i>0 then
					-- outerProgressScope:setPortionComplete(i/nbRenditions)
				-- end

				skipProcessing = (isInStack and not prefs.doBatchStacks) or (not isInStack and prefs.doBatchStacks)

				if skipProcessing then
					nbSkipped=nbSkipped+1
				else
					nbTreated=nbTreated+1
					-- outerProgressScope:setCaption(string.format("Importing %d named: %s",i, LrPathUtils.leafName(pathOrMessage)) )
					-- LrTasks.sleep(1)

					if not tempPath then -- no tempPath defined yet, or previous one was deleted
						tempPath=BatchUploadTask.getTempPath()
					end

					-- if not srcImagesFolder then
						-- -- srcImagesFolder = LrPathUtils.parent(pathOrMessage)
						-- srcImagesFolder = LrPathUtils.parent(pathOrMessage)
					-- end

					local copyPath = LrPathUtils.child(tempPath, LrPathUtils.leafName(pathOrMessage))
					LrFileUtils.copy(pathOrMessage, copyPath)
					table.insert(files, copyPath)
					nFiles = nFiles + 1
				end
				outerProgressScope:setPortionComplete(nbTreated/(nbToTreat+1))
			else
				LrErrors.throwUserError(LOC "$$$/Photomatix/Batch/Message/RenditionFailed=Lightroom rendition failed. Please check that the selected photos exist.")
				---	error(pathOrMessage)
			end

			-- if (nFiles<kMax) then
				-- continue
			-- end
			local continueExport=skipProcessing or (nFiles<kMax)

			if not continueExport then -- we have enough exported images for a bracketed set, so we process them

				LrFunctionContext.callWithContext('BatchOneSet', function(context)

                    local txtProgressProcess=string.format(LOC "$$$/Photomatix/Batch/Progress/Message/Bracketed=Merging %d photos starting from %s...",nFiles,LrPathUtils.leafName(LrPathUtils.removeExtension(files[1])))
                    if (kMax == 1) then
                        txtProgressProcess=string.format(LOC "$$$/Photomatix/Batch/Progress/Message/Single=Processing %s...",LrPathUtils.leafName(LrPathUtils.removeExtension(files[1])))
                    end

					-- local progressScope = LrDialogs.showModalProgressDialog({
						-- title = LOC "$$$/Photomatix/Batch/Progress/Title/BatchExport=Photomatix Batch Export",
						-- caption = txtProgressProcess,
						-- cannotCancel = true,
						-- parent=outerProgressScope,
						-- functionContext = context,
					-- })

					-- local innerProgressScope = LrProgressScope({
						-- caption = LOC "$$$/Photomatix/Batch/Progress/Title/BatchExport=Photomatix Batch Export",
						-- -- caption = string.format("Processing %d files starting with %s",nFiles,LrPathUtils.leafName(files[1])),
						-- caption = txtProgressProcess,
						-- cannotCancel = false,
						-- parent=outerProgressScope,
						-- functionContext = context,
					-- })
					outerProgressScope:setCaption(txtProgressProcess)

					-- innerProgressScope:setIndeterminate()

					LrTasks.yield()



					-- retour=LrTasks.execute(PhotomatixClientCommand)
					-- local processingSucceeded = BatchUploadTask.callPhotomatixCL(files, outFile, presetPath)
					local outTempPath = BatchUploadTask.getTempPath()
					local processingSucceeded, errMess = BatchUploadTask.callPhotomatixCL(files, outTempPath, presetPath)
					local pathToReimport = nil


					if processingSucceeded then
						activeCatalog = LrApplication.activeCatalog()

						local nOutImages = 0
						local resImageFile = nil
						for outFile in LrFileUtils.files(outTempPath) do
							nOutImages = nOutImages + 1
							resImageFile = outFile
						end

						if  not resImageFile or nOutImages ~= 1 then
							BatchUploadTask.outputToLog(string.format("Result image files not as expected: %s (nOutImages=%d)",resImageFile,nOutImages))
							errMess = LOC "$$$/Photomatix/Batch/Error/BadOutput=Couldn't access processed image"
							processingSucceeded=false
						else
							pathToReimport=LrPathUtils.child(srcImagesFolder,LrPathUtils.leafName(resImageFile))
							pathToReimport=LrFileUtils.chooseUniqueFileName(pathToReimport)
							BatchUploadTask.outputToLog(string.format("About to copy result image from %s to %s", resImageFile, pathToReimport))
							local copySuccess, reason=LrFileUtils.copy(resImageFile,pathToReimport)

							if not copySuccess then
								BatchUploadTask.outputToLog(string.format("Copy failed for reason=%s", reason))
								errMess = string.format(LOC "$$$/Photomatix/Batch/Error/CantCopy=Processed image copy failed: %s",reason)
								processingSucceeded = false
							end
						end
					end
					if processingSucceeded then --enter again same conidition which could have been put to false in code above

						local undoActionName=string.format(LOC "$$$/Photomatix/Batch/UndoActionName=Add %s to catalog", LrPathUtils.leafName(pathToReimport))

						if LrFileUtils.exists(pathToReimport)	then
							BatchUploadTask.outputToLog(string.format('reimporting file: %s',pathToReimport))
						else
							BatchUploadTask.outputToLog(string.format('file to reimport does not exist: %s',pathToReimport))
							errMess = string.format(LOC "$$$/Photomatix/Batch/Error/AccessFileReimport=Could not access reimported file: %s", pathToReimport)
							processingSucceeded = false
						end

						-- if (prefs.stackWith) then
							-- photoToStackWith=nil
							-- activeCatalog:withReadAccessDo( function()
								-- photoToStackWith = activeCatalog:findPhotoByPath(pathPhotoToStackWith)
							-- end )
						-- end

						-- See if the photo we are importing is already in the catalog
						local existingPhoto = nil
						activeCatalog:withReadAccessDo( function()
							existingPhoto = activeCatalog:findPhotoByPath(pathToReimport)
						end )


						local addedPhoto=nil
						local status, err
						BatchUploadTask.outputToLog(string.format("About to add images, photo to stack with=%s",photoToStackWith))
						if (photoToStackWith==nil) then
							activeCatalog:withWriteAccessDo( undoActionName, function()
								-- Work around bug in LR3 that raises an error when adding a photo that already exists
								status, err = LrTasks.pcall ( function()
									addedPhoto=activeCatalog:addPhoto(pathToReimport)
								end )
								-- Add into collection also if required
								if sourceCollection ~= nil then
									local photoArr = { activeCatalog:findPhotoByPath(pathToReimport) }
									sourceCollection:addPhotos(photoArr)
								end
							end )
						else
							activeCatalog:withWriteAccessDo( undoActionName, function()
								-- Work around bug in LR3 that raises an error when adding a photo that already exists
								status, err = LrTasks.pcall ( function()
									addedPhoto=activeCatalog:addPhoto(pathToReimport, photoToStackWith, 'below')
								end )
								if sourceCollection ~= nil then
									local photoArr = { activeCatalog:findPhotoByPath(pathToReimport) }
									sourceCollection:addPhotos(photoArr)
								end
							end )
						end


						-- If importing into
						if not status then
							if existingPhoto == nil then
								errMess = string.format(LOC "$$$/Photomatix/Batch/Error/WhileImporting=Error while importing photo: %s",  err)
								processingSucceeded = false
							-- else ignore error
							end
						end
					end
					-- Cleaning up source and dest directories
					if true then
						BatchUploadTask.outputToLog(string.format('Deleting src folder: %s',tempPath))
						srcDeleted, srcReason = LrFileUtils.delete(tempPath)
						BatchUploadTask.outputToLog(string.format('Deletion success: %s (%s)', srcDeleted, srcReason))
						BatchUploadTask.outputToLog(string.format('Deleting out folder: %s',outTempPath))
						dstDeleted, dstReason = LrFileUtils.delete(outTempPath)
						BatchUploadTask.outputToLog(string.format('Deletion success: %s (%s)', dstDeleted, dstReason))
					end
					tempPath = nil -- so that a new temp folder is generated for the next batch iteration
					if not processingSucceeded then
						nbErrors = nbErrors + 1
						if kMax == 1 then
							nbSingleErrors = nbSingleErrors + 1
						else
							nbBracketErrors = nbBracketErrors + 1
						end
						if errMess then
							table.insert(errorMessages, errMess)
						else
							LrErrors.throwUserError(string.format(LOC "$$$/Photomatix/Batch/Message/BatchCantProcess=HDR Batch could not process %s", LrPathUtils.leafName(LrPathUtils.removeExtension(files[1]))))
							return
						end
					else
						nbSuccess = nbSuccess + 1
						if kMax == 1 then
							nbSingleSuccess = nbSingleSuccess + 1
						else
							nbBracketSuccess = nbBracketSuccess + 1
						end
					end

					-- innerProgressScope:done()
				end)
				-- We reset the files tracking a given bracketed export
				files={}
				nFiles=0
				if prefs.doBatchStacks then
					kMax = 1 --we reset kMax to 1 as it could be more if last processed was a stack
				end
				srcImagesFolder = nil
				photoToStackWith = nil
			-- elseif skipProcessing then
			-- 	photoToStackWith = nil
			end

		end

		-- if prefs.isDebug then
			-- LrDialogs.message(string.format("processed=%d skipped=%d total=%d",nbTreated,nbSkipped,nbSkipped+nbTreated) )
		-- end

		outerProgressScope:done()

		local messTitle = LOC "$$$/Photomatix/Batch/Message/ProcessingSummary/CompleteSuccess=Batch successfully completed"
		-- if nbErrors > 0 or ((not prefs.doBatchStacks) and (nbToTreat==1 or not prefs.hasUnstacked)) then
		if nbErrors > 0 or (nbBracketSuccess == 0 and not prefs.doBatchStacks) then
			messTitle = LOC "$$$/Photomatix/Batch/Message/ProcessingSummary/CompleteWithErrors=Batch completed"
		end

		local messSummary = ""
		if nbBracketSuccess > 0 then
			local messBracket
			if nbBracketSuccess > 1 then
				messBracket = string.format(LOC "$$$/Photomatix/Batch/Message/ProcessingSummary/BracketProcessedPlural=%d bracketed photo sets processed.", nbBracketSuccess)
			else
				messBracket = LOC "$$$/Photomatix/Batch/Message/ProcessingSummary/BracketProcessedSingular=1 bracketed photo set processed."
			end
			messSummary = string.format("%s%s\n", messSummary, messBracket)
		end

		if nbSingleSuccess > 0 then
			local messSingle
			if nbSingleSuccess > 1 then
				messSingle = string.format(LOC "$$$/Photomatix/Batch/Message/ProcessingSummary/SingleProcessedPlural=%d single images processed.", nbSingleSuccess)
			else
				messSingle = LOC "$$$/Photomatix/Batch/Message/ProcessingSummary/SingleProcessedSingular=1 single image processed."
			end
			messSummary = string.format("%s%s\n", messSummary, messSingle)
		end

		if nbErrors > 0 then
			local messError
			if nbBracketErrors > 0 and nbSingleErrors > 0 then
				messError = string.format(LOC "$$$/Photomatix/Batch/Message/ProcessingSummary/ErrorBracketAndSingle=%d bracketed photo sets and %d single images couldn't be processed.\n\nError found: %s", nbBracketErrors, nbSingleErrors, errorMessages[1])
			elseif nbBracketErrors > 0 then
				messError = string.format(LOC "$$$/Photomatix/Batch/Message/ProcessingSummary/ErrorBracketOnly=%d bracketed photo sets couldn't be processed.\n\nError found: %s", nbBracketErrors, errorMessages[1])
			elseif nbSingleErrors > 0 then
				messError = string.format(LOC "$$$/Photomatix/Batch/Message/ProcessingSummary/ErrorSingleOnly=%d single images couldn't be processed.\n\nError found: %s", nbSingleErrors, errorMessages[1])
			end
			messSummary = string.format("%s\n%s", messSummary, messError)
		end
		if nbSkipped > 0 then
			local errSkipped = LOC "$$$/Photomatix/Batch/Message/ProcessingSummary/StacksSkipped=%d stacks were ignored."
			if (prefs.doBatchStacks) then
				errSkipped = LOC "$$$/Photomatix/Batch/Message/ProcessingSummary/SingleSkipped=%d photos not in stacks were ignored."
			end
			if (nbSkipped == 1) then
				errSkipped=LOC "$$$/Photomatix/Batch/Message/ProcessingSummary/OneStackSkipped=%d stack was ignored."
				if (prefs.doBatchStacks) then
					errSkipped=LOC "$$$/Photomatix/Batch/Message/ProcessingSummary/OneSingleSkipped=%d photo not in stack was ignored."
				end
			end
			messSummary = string.format("%s\n%s", messSummary, string.format(errSkipped, nbSkipped))
		end
		LrDialogs.message(messTitle, messSummary)


	end)  -- End of section done while showing progress dialog
end


pmpclArgs=nil

function BatchUploadTask.buildStaticPhotomatixCLArguments(nbPhotos)

	pmpclArgs=""
	if WIN_ENV then -- -h option doesn't exist for Mac version
		pmpclArgs=string.format("%s -h remove",pmpclArgs)
	end
	local bpsOut=8
	local frmOut="jpg"
	if prefs.outputFormat=="tif8" or prefs.outputFormat=="tif16"  then
		frmOut="tif"
		if prefs.outputFormat=="tif16" then
			bpsOut=16
		end
	end
	pmpclArgs=string.format("%s -s %s -bi %d", pmpclArgs, frmOut, bpsOut)

	-- pmpclArgs=string.format("%s -s tif",pmpclArgs)

	if prefs.cbxContrast>0 then
		local ch, cl, cd, cs = 0, 0, 0, 0
		if prefs.cbxContrast==1 then
			ch, cs = 10, -15
		elseif prefs.cbxContrast==2 then
			ch, cs = 15, -25
		elseif prefs.cbxContrast==3 then
			ch, cs = 25, -50
		elseif prefs.cbxContrast==4 then
			ch, cl, cd, cs = 50, 25, -25, -75
		elseif prefs.cbxContrast==5 then
			ch, cl, cd, cs = 75, 50, -50, -100
		end
		pmpclArgs=string.format("%s -ch %d -cl %d -cd %d -cs %d", pmpclArgs, ch, cl, cd, cs)
	end

	if prefs.cbxSharpening>0 then
		local sa, sr, st = 0, 0, 0
		if prefs.cbxSharpening==1 then
			sa, sr, st = 50, 1, 5
		elseif prefs.cbxSharpening==2 then
			sa, sr, st = 100, 2, 5
		elseif prefs.cbxSharpening==3 then
			sa, sr, st = 150, 3, 2
		end
		pmpclArgs=string.format("%s -sa %d -sr %d -st %d", pmpclArgs, sa, sr, st)
	end

	if prefs.chkAlign and nbPhotos > 1 then
		local alignType = prefs.presetAlign
		local maxShift=0
		if alignType == "0" then -- Hand-held method large shifts
			alignType="2" -- The method to use is Hand-held not 0 which doesn't exists and was just used to trigger this code
			maxShift = 20 --large shifts
		end
		if prefs.chkCrop then
			pmpclArgs=string.format('%s -a%s', pmpclArgs, alignType)
		else
			pmpclArgs=string.format('%s -a%sn', pmpclArgs, alignType)
		end
		if maxShift>0 then
			pmpclArgs=string.format('%s -am %d', pmpclArgs, maxShift)
		end
	end

-- Naming options:

  if false and WIN_ENV then
  	if prefs.fileNameSuffix and prefs.fileNameSuffix ~= "" and #prefs.fileNameSuffix>0 then
  		pmpclArgs=string.format("%s -ns %d %s", pmpclArgs, prefs.outputFileNameOption, prefs.fileNameSuffix)
  	else
  		pmpclArgs=string.format("%s -n %d", pmpclArgs, prefs.outputFileNameOption)
  	end
  end

--	if prefs.isDebug and WIN_ENV then
--		pmpclArgs=string.format("%s -z",pmpclArgs)
--	end

	if prefs.chkGhosting and nbPhotos > 1 then
		pmpclArgs=string.format("%s -gn",pmpclArgs)
	end

end

function BatchUploadTask.testDialog(propertyTable)
	LrDialogs.message("In StartDialog")
end

function BatchUploadTask.testDialog2(propertyTable)
	LrDialogs.message("In EndDialog")
end

-- function BatchUploadTask.callPhotomatixCL(inputFiles, outFile, presetPath)
function BatchUploadTask.callPhotomatixCL(inputFiles, outPath, presetPath)

	local PhotomatixClientCommand
	-- if pmpclArgs==nil then --now we call buildStaticPhotomatixCLArguments each time because parameters could be different if processing stacks and single photos
	BatchUploadTask.buildStaticPhotomatixCLArguments(#inputFiles)
		-- BatchUploadTask.outputToLog(string.format("\n\npmpclArgs was null, now it is: %s\n\n",pmpclArgs))
	-- else
		-- BatchUploadTask.outputToLog(string.format("pmpclArgs not null, stays: %s",pmpclArgs))
	-- end

	local namingFunction=BatchInputSettings.getDefaultFileName
	if (prefs.outputFileNameOption=='1') then
		namingFunction=BatchInputSettings.getFirstFileName
	end
	local outFilExt="jpg"
	if prefs.outputFormat ~= "jpg" then
		outFilExt="tif"
	end
	local outFileName=namingFunction(inputFiles)
	if prefs.fileNameSuffix and prefs.fileNameSuffix ~= "" and #prefs.fileNameSuffix>0 then
		outFileName=string.format("%s%s",outFileName,prefs.fileNameSuffix)
	end
	local outFileNameWithExt=LrPathUtils.addExtension(outFileName,outFilExt)
	local outFileNameFull=LrPathUtils.child(outPath,outFileNameWithExt)
	BatchUploadTask.outputToLog(string.format("Out filename is: %s",outFileName))


	if WIN_ENV then
		-- Generate key to pass to PhotomatixCL so validate it's being called from the plugin
		local currClock = os.time(os.date('*t'))
		local clock1 = currClock
		-- currClock = currClock & 0xffffffff
			for pow = 63,32,-1 do
					if currClock - (2 ^ pow) > 0 then
						currClock = i - (2 ^ pow)
					end
			end
			currClock = math.floor(currClock / 10)

		clCode = 0
		local utilAppPath=BatchUploadTask.getBatchPathUtil()
		if (WIN_ENV and utilAppPath) then
			local CommandLine=string.format('\"%s\" -kk %s %s', utilAppPath, machID, currClock)
			CommandLine=string.format('\"%s\"', CommandLine)
			BatchUploadTask.outputToLog(string.format('Command is: %s', CommandLine))
			clCode=LrTasks.execute(CommandLine)
			BatchUploadTask.outputToLog(string.format('After LrTasks.execute - retour is: %s', clCode))
		end

		local clock2 = os.time(os.date('*t'))
		BatchUploadTask.outputToLog(string.format("Time to do first part: %d", (clock2-clock1)))

		PhotomatixClientCommand=string.format('\"%s\"%s -Z %x -x \"%s\" -o \"%s\" -d \"%s\\\\\"', appPath, pmpclArgs, clCode, presetPath, outFileName, outPath)
	else
		PhotomatixClientCommand=string.format('\"%s\"%s -x \"%s\" -o \"%s\"', appPath, pmpclArgs, presetPath, outFileNameFull)
	end

	outputTempFile=string.format('%sCmdOutput', tempPath)
	local inputFilesArgs=""
	for k,fn in ipairs(inputFiles) do
		inputFilesArgs=string.format('%s \"%s\"',inputFilesArgs,fn)
	end

	if (WIN_ENV) then
		PhotomatixClientCommand=string.format('%s %s', PhotomatixClientCommand, inputFilesArgs)
		-- LrDialogs.message("About to execute command: "..PhotomatixClientCommand)
		PhotomatixClientCommand=string.format('%s 1> \"%s\" 2>&1', PhotomatixClientCommand, outputTempFile)
		PhotomatixClientCommand=string.format('\"%s\"', PhotomatixClientCommand)
	else
		PhotomatixClientCommand=string.format('%s -i%d %s', PhotomatixClientCommand, #inputFiles, inputFilesArgs)
	end

	BatchUploadTask.outputToLog(string.format('PhotomatixClientCommand: %s', PhotomatixClientCommand))

	retour=LrTasks.execute(PhotomatixClientCommand)
	BatchUploadTask.outputToLog(string.format('retour PhotomatixClient: %d', retour))

	-- if photosProcessed then


			-- -- Remove temp files, because PhotomatixCL does not do this.
			-- if WIN_ENV then
				-- if string.find(tempPath, "lrexport2photomatix") ~= nil then  -- Make extra-sure that this really is a temp path
					-- LrFileUtils.delete(tempPath)
				-- end
			-- end
-- -------------------------- Returning from command-line program ------------------------------------

	local processingSucceeded = true
	if WIN_ENV then
		local outputMessage
		if math.abs(retour) >= 0x40000000 and math.abs(retour) <= 0xff000000 and WIN_ENV then
			-- These seem to correspond to PhotomatiCL crashing on Windows, so we are not likely to get a sensible output message
			outputMessage = LOC "$$$/Photomatix/Batch/Message/DefaultProcessingFail=Processing failed"
			processingSucceeded = false
		elseif retour <= 0 or retour > 0xff000000 then
			processingSucceeded = false
			-- PhotomatixCL finished, but returned an error code
			local handle = io.open(outputTempFile, 'rb')
			if handle then
				BatchUploadTask.outputToLog("Handle ok", true)
				while true do
					local currLine = handle:read('*line')
					if currLine == nil then break end
					-- if outputMessage == nil then
						outputMessage = currLine
					-- else
						-- outputMessage = string.format("%s\n%s", outputMessage, currLine)
					-- end
				end
				handle:close()
			end
		end

		if outputMessage == nil or string.find(outputMessage, "^%d+$") then  -- If just a number, there was no final message, and we just got one of the normal codes printed by PhotomatixCL
			outputMessage = ""
			if retour == 0xfffffffc then  -- -4: out of memory, and no custom error message was displayed
				outputMessage=LOC "$$$/Photomatix/General/Error/OutOfMemory=There was insufficient memory to process your images"
			end
		end

		local formattedMsg = string.format('%s',outputMessage)
		BatchUploadTask.outputToLog(string.format('Photomatix CL processingSucceeded=%s output message: %s',processingSucceeded, formattedMsg))

		if not processingSucceeded then
				-- LrDialogs.message("Error while exporting: " .. outputMessage, nil, "critical")
				return processingSucceeded, outputMessage
			end
		else
			-- MAC_ENV
			if retour ~= 0 then processingSucceeded = false end
		end

		return processingSucceeded

end

